# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.feature_collection import FeatureCollection
from ..types.time_series_feature_collection_model import TimeSeriesFeatureCollectionModel
from .raw_client import AsyncRawPortfoliosClient, RawPortfoliosClient


class PortfoliosClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPortfoliosClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPortfoliosClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPortfoliosClient
        """
        return self._raw_client

    def features(
        self,
        portfolio_id: str,
        *,
        x: typing.Optional[int] = None,
        y: typing.Optional[int] = None,
        z: typing.Optional[int] = None,
        country: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeatureCollection:
        """
        Return a GeoJSON FeatureCollection of features for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        x : typing.Optional[int]

        y : typing.Optional[int]

        z : typing.Optional[int]

        country : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeatureCollection
            Successful Response

        Examples
        --------
        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.portfolios.features(
            portfolio_id="portfolio_id",
            x=1,
            y=1,
            z=1,
            country="country",
        )
        """
        _response = self._raw_client.features(
            portfolio_id, x=x, y=y, z=z, country=country, request_options=request_options
        )
        return _response.data

    def forecasts(
        self,
        portfolio_id: str,
        *,
        init_time: dt.datetime,
        id: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        variables: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TimeSeriesFeatureCollectionModel:
        """
        Return a GeoJSON FeatureCollection of forecasts for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        init_time : dt.datetime

        id : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        variables : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TimeSeriesFeatureCollectionModel
            Successful Response

        Examples
        --------
        import datetime

        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.portfolios.forecasts(
            portfolio_id="portfolio_id",
            init_time=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
        )
        """
        _response = self._raw_client.forecasts(
            portfolio_id, init_time=init_time, id=id, variables=variables, request_options=request_options
        )
        return _response.data

    def observations(
        self,
        portfolio_id: str,
        *,
        valid_time_start: dt.datetime,
        valid_time_end: dt.datetime,
        id: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        variables: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TimeSeriesFeatureCollectionModel:
        """
        Return a GeoJSON FeatureCollection of observations for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        valid_time_start : dt.datetime

        valid_time_end : dt.datetime

        id : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        variables : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TimeSeriesFeatureCollectionModel
            Successful Response

        Examples
        --------
        import datetime

        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.portfolios.observations(
            portfolio_id="portfolio_id",
            valid_time_start=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
            valid_time_end=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
        )
        """
        _response = self._raw_client.observations(
            portfolio_id,
            valid_time_start=valid_time_start,
            valid_time_end=valid_time_end,
            id=id,
            variables=variables,
            request_options=request_options,
        )
        return _response.data

    def init_time(
        self, portfolio_id: str, *, time: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> dt.datetime:
        """
        Return the latest initialization time for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        time : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        dt.datetime
            Successful Response

        Examples
        --------
        import datetime

        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.portfolios.init_time(
            portfolio_id="portfolio_id",
            time=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
        )
        """
        _response = self._raw_client.init_time(portfolio_id, time=time, request_options=request_options)
        return _response.data


class AsyncPortfoliosClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPortfoliosClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPortfoliosClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPortfoliosClient
        """
        return self._raw_client

    async def features(
        self,
        portfolio_id: str,
        *,
        x: typing.Optional[int] = None,
        y: typing.Optional[int] = None,
        z: typing.Optional[int] = None,
        country: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeatureCollection:
        """
        Return a GeoJSON FeatureCollection of features for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        x : typing.Optional[int]

        y : typing.Optional[int]

        z : typing.Optional[int]

        country : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeatureCollection
            Successful Response

        Examples
        --------
        import asyncio

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.portfolios.features(
                portfolio_id="portfolio_id",
                x=1,
                y=1,
                z=1,
                country="country",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.features(
            portfolio_id, x=x, y=y, z=z, country=country, request_options=request_options
        )
        return _response.data

    async def forecasts(
        self,
        portfolio_id: str,
        *,
        init_time: dt.datetime,
        id: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        variables: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TimeSeriesFeatureCollectionModel:
        """
        Return a GeoJSON FeatureCollection of forecasts for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        init_time : dt.datetime

        id : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        variables : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TimeSeriesFeatureCollectionModel
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.portfolios.forecasts(
                portfolio_id="portfolio_id",
                init_time=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.forecasts(
            portfolio_id, init_time=init_time, id=id, variables=variables, request_options=request_options
        )
        return _response.data

    async def observations(
        self,
        portfolio_id: str,
        *,
        valid_time_start: dt.datetime,
        valid_time_end: dt.datetime,
        id: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        variables: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TimeSeriesFeatureCollectionModel:
        """
        Return a GeoJSON FeatureCollection of observations for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        valid_time_start : dt.datetime

        valid_time_end : dt.datetime

        id : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        variables : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TimeSeriesFeatureCollectionModel
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.portfolios.observations(
                portfolio_id="portfolio_id",
                valid_time_start=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                valid_time_end=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.observations(
            portfolio_id,
            valid_time_start=valid_time_start,
            valid_time_end=valid_time_end,
            id=id,
            variables=variables,
            request_options=request_options,
        )
        return _response.data

    async def init_time(
        self, portfolio_id: str, *, time: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> dt.datetime:
        """
        Return the latest initialization time for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        time : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        dt.datetime
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.portfolios.init_time(
                portfolio_id="portfolio_id",
                time=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.init_time(portfolio_id, time=time, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from .time_series_feature_model_geometry import TimeSeriesFeatureModelGeometry
from .time_series_feature_model_id import TimeSeriesFeatureModelId
from .time_series_properties_model import TimeSeriesPropertiesModel


class TimeSeriesFeatureModel(UniversalBaseModel):
    """
    GeoJSON Feature with time-series properties.
    """

    bbox: typing.Optional[typing.List[typing.Optional[typing.Any]]] = None
    type: typing.Literal["Feature"] = "Feature"
    geometry: typing.Optional[TimeSeriesFeatureModelGeometry] = None
    properties: typing.Optional[TimeSeriesPropertiesModel] = None
    id: typing.Optional[TimeSeriesFeatureModelId] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(TimeSeriesFeatureModel)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.cyclone_forecast_response import CycloneForecastResponse
from ...types.feature_collection import FeatureCollection
from ...types.model_name import ModelName
from .raw_client import AsyncRawForecastsClient, RawForecastsClient


class ForecastsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawForecastsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawForecastsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawForecastsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        time: typing.Optional[dt.datetime] = None,
        min_storm_category: typing.Optional[int] = None,
        basin_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        model: typing.Optional[ModelName] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[CycloneForecastResponse]:
        """
        Query the available cyclone forecasts for a particular time

        Parameters
        ----------
        time : typing.Optional[dt.datetime]
            *Default value: (current time) - Default time zone: UTC*

        min_storm_category : typing.Optional[int]

        basin_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        model : typing.Optional[ModelName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[CycloneForecastResponse]
            Successful Response

        Examples
        --------
        import datetime

        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.cyclones.forecasts.list(
            time=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
            min_storm_category=1,
            model="OFCL",
        )
        """
        _response = self._raw_client.list(
            time=time,
            min_storm_category=min_storm_category,
            basin_ids=basin_ids,
            model=model,
            request_options=request_options,
        )
        return _response.data

    def track(
        self,
        storm_id: str,
        *,
        time: typing.Optional[dt.datetime] = None,
        max_lead_time: typing.Optional[str] = None,
        model: typing.Optional[ModelName] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeatureCollection:
        """
        Get cyclone tracks in GeoJSON (MF-GeoJSON) format

        Parameters
        ----------
        storm_id : str

        time : typing.Optional[dt.datetime]
            *Default value: (current time) - Default time zone: UTC*

        max_lead_time : typing.Optional[str]
            *Value must be > P0D*

        model : typing.Optional[ModelName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeatureCollection
            Successful Response

        Examples
        --------
        import datetime

        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.cyclones.forecasts.track(
            storm_id="storm_id",
            time=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
            max_lead_time="max_lead_time",
            model="OFCL",
        )
        """
        _response = self._raw_client.track(
            storm_id, time=time, max_lead_time=max_lead_time, model=model, request_options=request_options
        )
        return _response.data

    def cone(
        self,
        storm_id: str,
        *,
        time: typing.Optional[dt.datetime] = None,
        max_lead_time: typing.Optional[str] = None,
        smooth_cone: typing.Optional[bool] = None,
        model: typing.Optional[ModelName] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeatureCollection:
        """
        Get cyclone forecast cone in GeoJSON format

        Parameters
        ----------
        storm_id : str

        time : typing.Optional[dt.datetime]
            *Default value: (current time) - Default time zone: UTC*

        max_lead_time : typing.Optional[str]
            *Value must be > P0D*

        smooth_cone : typing.Optional[bool]

        model : typing.Optional[ModelName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeatureCollection
            Successful Response

        Examples
        --------
        import datetime

        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.cyclones.forecasts.cone(
            storm_id="storm_id",
            time=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
            max_lead_time="max_lead_time",
            smooth_cone=True,
            model="OFCL",
        )
        """
        _response = self._raw_client.cone(
            storm_id,
            time=time,
            max_lead_time=max_lead_time,
            smooth_cone=smooth_cone,
            model=model,
            request_options=request_options,
        )
        return _response.data


class AsyncForecastsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawForecastsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawForecastsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawForecastsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        time: typing.Optional[dt.datetime] = None,
        min_storm_category: typing.Optional[int] = None,
        basin_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        model: typing.Optional[ModelName] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[CycloneForecastResponse]:
        """
        Query the available cyclone forecasts for a particular time

        Parameters
        ----------
        time : typing.Optional[dt.datetime]
            *Default value: (current time) - Default time zone: UTC*

        min_storm_category : typing.Optional[int]

        basin_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        model : typing.Optional[ModelName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[CycloneForecastResponse]
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.cyclones.forecasts.list(
                time=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                min_storm_category=1,
                model="OFCL",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            time=time,
            min_storm_category=min_storm_category,
            basin_ids=basin_ids,
            model=model,
            request_options=request_options,
        )
        return _response.data

    async def track(
        self,
        storm_id: str,
        *,
        time: typing.Optional[dt.datetime] = None,
        max_lead_time: typing.Optional[str] = None,
        model: typing.Optional[ModelName] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeatureCollection:
        """
        Get cyclone tracks in GeoJSON (MF-GeoJSON) format

        Parameters
        ----------
        storm_id : str

        time : typing.Optional[dt.datetime]
            *Default value: (current time) - Default time zone: UTC*

        max_lead_time : typing.Optional[str]
            *Value must be > P0D*

        model : typing.Optional[ModelName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeatureCollection
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.cyclones.forecasts.track(
                storm_id="storm_id",
                time=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                max_lead_time="max_lead_time",
                model="OFCL",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.track(
            storm_id, time=time, max_lead_time=max_lead_time, model=model, request_options=request_options
        )
        return _response.data

    async def cone(
        self,
        storm_id: str,
        *,
        time: typing.Optional[dt.datetime] = None,
        max_lead_time: typing.Optional[str] = None,
        smooth_cone: typing.Optional[bool] = None,
        model: typing.Optional[ModelName] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeatureCollection:
        """
        Get cyclone forecast cone in GeoJSON format

        Parameters
        ----------
        storm_id : str

        time : typing.Optional[dt.datetime]
            *Default value: (current time) - Default time zone: UTC*

        max_lead_time : typing.Optional[str]
            *Value must be > P0D*

        smooth_cone : typing.Optional[bool]

        model : typing.Optional[ModelName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeatureCollection
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.cyclones.forecasts.cone(
                storm_id="storm_id",
                time=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                max_lead_time="max_lead_time",
                smooth_cone=True,
                model="OFCL",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cone(
            storm_id,
            time=time,
            max_lead_time=max_lead_time,
            smooth_cone=smooth_cone,
            model=model,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.daily_weather_response import DailyWeatherResponse
from ...types.hourly_weather_response import HourlyWeatherResponse
from ...types.timezone import Timezone
from ...types.units import Units
from .raw_client import AsyncRawForecastClient, RawForecastClient


class ForecastClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawForecastClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawForecastClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawForecastClient
        """
        return self._raw_client

    def daily(
        self,
        *,
        latitude: float,
        longitude: float,
        timezone: typing.Optional[typing.Literal["local"]] = None,
        units: typing.Optional[Units] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DailyWeatherResponse:
        """
        Get daily weather forecast for a specific location
        Only allowing local timezone aggregations for now since
        it is unclear how exactly users will understand "UTC".

        Parameters
        ----------
        latitude : float

        longitude : float

        timezone : typing.Optional[typing.Literal["local"]]

        units : typing.Optional[Units]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DailyWeatherResponse
            Successful Response

        Examples
        --------
        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.weather.forecast.daily(
            latitude=47.6061,
            longitude=-122.3328,
            units="metric",
        )
        """
        _response = self._raw_client.daily(
            latitude=latitude, longitude=longitude, timezone=timezone, units=units, request_options=request_options
        )
        return _response.data

    def hourly(
        self,
        *,
        latitude: float,
        longitude: float,
        timezone: typing.Optional[Timezone] = None,
        units: typing.Optional[Units] = None,
        include_past: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HourlyWeatherResponse:
        """
        Get hourly weather forecast for a specific location

        Parameters
        ----------
        latitude : float

        longitude : float

        timezone : typing.Optional[Timezone]

        units : typing.Optional[Units]

        include_past : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HourlyWeatherResponse
            Successful Response

        Examples
        --------
        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.weather.forecast.hourly(
            latitude=47.6061,
            longitude=-122.3328,
            timezone="local",
            units="metric",
            include_past=True,
        )
        """
        _response = self._raw_client.hourly(
            latitude=latitude,
            longitude=longitude,
            timezone=timezone,
            units=units,
            include_past=include_past,
            request_options=request_options,
        )
        return _response.data


class AsyncForecastClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawForecastClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawForecastClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawForecastClient
        """
        return self._raw_client

    async def daily(
        self,
        *,
        latitude: float,
        longitude: float,
        timezone: typing.Optional[typing.Literal["local"]] = None,
        units: typing.Optional[Units] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DailyWeatherResponse:
        """
        Get daily weather forecast for a specific location
        Only allowing local timezone aggregations for now since
        it is unclear how exactly users will understand "UTC".

        Parameters
        ----------
        latitude : float

        longitude : float

        timezone : typing.Optional[typing.Literal["local"]]

        units : typing.Optional[Units]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DailyWeatherResponse
            Successful Response

        Examples
        --------
        import asyncio

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.weather.forecast.daily(
                latitude=47.6061,
                longitude=-122.3328,
                units="metric",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.daily(
            latitude=latitude, longitude=longitude, timezone=timezone, units=units, request_options=request_options
        )
        return _response.data

    async def hourly(
        self,
        *,
        latitude: float,
        longitude: float,
        timezone: typing.Optional[Timezone] = None,
        units: typing.Optional[Units] = None,
        include_past: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HourlyWeatherResponse:
        """
        Get hourly weather forecast for a specific location

        Parameters
        ----------
        latitude : float

        longitude : float

        timezone : typing.Optional[Timezone]

        units : typing.Optional[Units]

        include_past : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HourlyWeatherResponse
            Successful Response

        Examples
        --------
        import asyncio

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.weather.forecast.hourly(
                latitude=47.6061,
                longitude=-122.3328,
                timezone="local",
                units="metric",
                include_past=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.hourly(
            latitude=latitude,
            longitude=longitude,
            timezone=timezone,
            units=units,
            include_past=include_past,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        BaseModel,
        CycloneForecastResponse,
        DailyConditions,
        DailyWeatherResponse,
        DailyWeatherResponseUnits,
        Feature,
        FeatureCollection,
        FeatureGeometry,
        FeatureGeometry_GeometryCollection,
        FeatureGeometry_LineString,
        FeatureGeometry_MultiLineString,
        FeatureGeometry_MultiPoint,
        FeatureGeometry_MultiPolygon,
        FeatureGeometry_Point,
        FeatureGeometry_Polygon,
        FeatureId,
        FeatureProperties,
        GeoLocation,
        GeometryCollection,
        GeometryCollectionGeometriesItem,
        GeometryCollectionGeometriesItem_GeometryCollection,
        GeometryCollectionGeometriesItem_LineString,
        GeometryCollectionGeometriesItem_MultiLineString,
        GeometryCollectionGeometriesItem_MultiPoint,
        GeometryCollectionGeometriesItem_MultiPolygon,
        GeometryCollectionGeometriesItem_Point,
        GeometryCollectionGeometriesItem_Polygon,
        GftusBaseUnits,
        GftusHourlyConditions,
        GftusHourlyWeatherResponse,
        HourlyConditions,
        HourlyWeatherResponse,
        HourlyWeatherResponseUnits,
        HttpValidationError,
        ImperialUnits,
        LineString,
        LineStringCoordinatesItem,
        MetricUnits,
        ModelName,
        MultiLineString,
        MultiLineStringCoordinatesItemItem,
        MultiPoint,
        MultiPointCoordinatesItem,
        MultiPolygon,
        MultiPolygonCoordinatesItemItemItem,
        Point,
        PointCoordinates,
        Polygon,
        PolygonCoordinatesItemItem,
        Position2D,
        Position3D,
        PrecipitationType,
        TimeSeriesFeatureCollectionModel,
        TimeSeriesFeatureModel,
        TimeSeriesFeatureModelGeometry,
        TimeSeriesFeatureModelGeometry_GeometryCollection,
        TimeSeriesFeatureModelGeometry_LineString,
        TimeSeriesFeatureModelGeometry_MultiLineString,
        TimeSeriesFeatureModelGeometry_MultiPoint,
        TimeSeriesFeatureModelGeometry_MultiPolygon,
        TimeSeriesFeatureModelGeometry_Point,
        TimeSeriesFeatureModelGeometry_Polygon,
        TimeSeriesFeatureModelId,
        TimeSeriesPropertiesModel,
        TimeSeriesPropertiesModelStatic,
        Timezone,
        Units,
        ValidationError,
        ValidationErrorLocItem,
        WeatherCode,
    )
    from .errors import UnprocessableEntityError
    from . import cyclones, portfolios, weather
    from .client import AsyncEarth, Earth
    from .environment import EarthEnvironment
    from .version import __version__
_dynamic_imports: typing.Dict[str, str] = {
    "AsyncEarth": ".client",
    "BaseModel": ".types",
    "CycloneForecastResponse": ".types",
    "DailyConditions": ".types",
    "DailyWeatherResponse": ".types",
    "DailyWeatherResponseUnits": ".types",
    "Earth": ".client",
    "EarthEnvironment": ".environment",
    "Feature": ".types",
    "FeatureCollection": ".types",
    "FeatureGeometry": ".types",
    "FeatureGeometry_GeometryCollection": ".types",
    "FeatureGeometry_LineString": ".types",
    "FeatureGeometry_MultiLineString": ".types",
    "FeatureGeometry_MultiPoint": ".types",
    "FeatureGeometry_MultiPolygon": ".types",
    "FeatureGeometry_Point": ".types",
    "FeatureGeometry_Polygon": ".types",
    "FeatureId": ".types",
    "FeatureProperties": ".types",
    "GeoLocation": ".types",
    "GeometryCollection": ".types",
    "GeometryCollectionGeometriesItem": ".types",
    "GeometryCollectionGeometriesItem_GeometryCollection": ".types",
    "GeometryCollectionGeometriesItem_LineString": ".types",
    "GeometryCollectionGeometriesItem_MultiLineString": ".types",
    "GeometryCollectionGeometriesItem_MultiPoint": ".types",
    "GeometryCollectionGeometriesItem_MultiPolygon": ".types",
    "GeometryCollectionGeometriesItem_Point": ".types",
    "GeometryCollectionGeometriesItem_Polygon": ".types",
    "GftusBaseUnits": ".types",
    "GftusHourlyConditions": ".types",
    "GftusHourlyWeatherResponse": ".types",
    "HourlyConditions": ".types",
    "HourlyWeatherResponse": ".types",
    "HourlyWeatherResponseUnits": ".types",
    "HttpValidationError": ".types",
    "ImperialUnits": ".types",
    "LineString": ".types",
    "LineStringCoordinatesItem": ".types",
    "MetricUnits": ".types",
    "ModelName": ".types",
    "MultiLineString": ".types",
    "MultiLineStringCoordinatesItemItem": ".types",
    "MultiPoint": ".types",
    "MultiPointCoordinatesItem": ".types",
    "MultiPolygon": ".types",
    "MultiPolygonCoordinatesItemItemItem": ".types",
    "Point": ".types",
    "PointCoordinates": ".types",
    "Polygon": ".types",
    "PolygonCoordinatesItemItem": ".types",
    "Position2D": ".types",
    "Position3D": ".types",
    "PrecipitationType": ".types",
    "TimeSeriesFeatureCollectionModel": ".types",
    "TimeSeriesFeatureModel": ".types",
    "TimeSeriesFeatureModelGeometry": ".types",
    "TimeSeriesFeatureModelGeometry_GeometryCollection": ".types",
    "TimeSeriesFeatureModelGeometry_LineString": ".types",
    "TimeSeriesFeatureModelGeometry_MultiLineString": ".types",
    "TimeSeriesFeatureModelGeometry_MultiPoint": ".types",
    "TimeSeriesFeatureModelGeometry_MultiPolygon": ".types",
    "TimeSeriesFeatureModelGeometry_Point": ".types",
    "TimeSeriesFeatureModelGeometry_Polygon": ".types",
    "TimeSeriesFeatureModelId": ".types",
    "TimeSeriesPropertiesModel": ".types",
    "TimeSeriesPropertiesModelStatic": ".types",
    "Timezone": ".types",
    "Units": ".types",
    "UnprocessableEntityError": ".errors",
    "ValidationError": ".types",
    "ValidationErrorLocItem": ".types",
    "WeatherCode": ".types",
    "__version__": ".version",
    "cyclones": ".cyclones",
    "portfolios": ".portfolios",
    "weather": ".weather",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AsyncEarth",
    "BaseModel",
    "CycloneForecastResponse",
    "DailyConditions",
    "DailyWeatherResponse",
    "DailyWeatherResponseUnits",
    "Earth",
    "EarthEnvironment",
    "Feature",
    "FeatureCollection",
    "FeatureGeometry",
    "FeatureGeometry_GeometryCollection",
    "FeatureGeometry_LineString",
    "FeatureGeometry_MultiLineString",
    "FeatureGeometry_MultiPoint",
    "FeatureGeometry_MultiPolygon",
    "FeatureGeometry_Point",
    "FeatureGeometry_Polygon",
    "FeatureId",
    "FeatureProperties",
    "GeoLocation",
    "GeometryCollection",
    "GeometryCollectionGeometriesItem",
    "GeometryCollectionGeometriesItem_GeometryCollection",
    "GeometryCollectionGeometriesItem_LineString",
    "GeometryCollectionGeometriesItem_MultiLineString",
    "GeometryCollectionGeometriesItem_MultiPoint",
    "GeometryCollectionGeometriesItem_MultiPolygon",
    "GeometryCollectionGeometriesItem_Point",
    "GeometryCollectionGeometriesItem_Polygon",
    "GftusBaseUnits",
    "GftusHourlyConditions",
    "GftusHourlyWeatherResponse",
    "HourlyConditions",
    "HourlyWeatherResponse",
    "HourlyWeatherResponseUnits",
    "HttpValidationError",
    "ImperialUnits",
    "LineString",
    "LineStringCoordinatesItem",
    "MetricUnits",
    "ModelName",
    "MultiLineString",
    "MultiLineStringCoordinatesItemItem",
    "MultiPoint",
    "MultiPointCoordinatesItem",
    "MultiPolygon",
    "MultiPolygonCoordinatesItemItemItem",
    "Point",
    "PointCoordinates",
    "Polygon",
    "PolygonCoordinatesItemItem",
    "Position2D",
    "Position3D",
    "PrecipitationType",
    "TimeSeriesFeatureCollectionModel",
    "TimeSeriesFeatureModel",
    "TimeSeriesFeatureModelGeometry",
    "TimeSeriesFeatureModelGeometry_GeometryCollection",
    "TimeSeriesFeatureModelGeometry_LineString",
    "TimeSeriesFeatureModelGeometry_MultiLineString",
    "TimeSeriesFeatureModelGeometry_MultiPoint",
    "TimeSeriesFeatureModelGeometry_MultiPolygon",
    "TimeSeriesFeatureModelGeometry_Point",
    "TimeSeriesFeatureModelGeometry_Polygon",
    "TimeSeriesFeatureModelId",
    "TimeSeriesPropertiesModel",
    "TimeSeriesPropertiesModelStatic",
    "Timezone",
    "Units",
    "UnprocessableEntityError",
    "ValidationError",
    "ValidationErrorLocItem",
    "WeatherCode",
    "__version__",
    "cyclones",
    "portfolios",
    "weather",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawPastClient, RawPastClient

if typing.TYPE_CHECKING:
    from .forecast.client import AsyncForecastClient, ForecastClient


class PastClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPastClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._forecast: typing.Optional[ForecastClient] = None

    @property
    def with_raw_response(self) -> RawPastClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPastClient
        """
        return self._raw_client

    @property
    def forecast(self):
        if self._forecast is None:
            from .forecast.client import ForecastClient  # noqa: E402

            self._forecast = ForecastClient(client_wrapper=self._client_wrapper)
        return self._forecast


class AsyncPastClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPastClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._forecast: typing.Optional[AsyncForecastClient] = None

    @property
    def with_raw_response(self) -> AsyncRawPastClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPastClient
        """
        return self._raw_client

    @property
    def forecast(self):
        if self._forecast is None:
            from .forecast.client import AsyncForecastClient  # noqa: E402

            self._forecast = AsyncForecastClient(client_wrapper=self._client_wrapper)
        return self._forecast



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .time_series_properties_model_static import TimeSeriesPropertiesModelStatic


class TimeSeriesPropertiesModel(UniversalBaseModel):
    """
    Concrete model for OpenAPI schema generation.
    """

    datetimes: typing.List[dt.datetime]
    static: TimeSeriesPropertiesModelStatic

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawCyclonesClient, RawCyclonesClient

if typing.TYPE_CHECKING:
    from .forecasts.client import AsyncForecastsClient, ForecastsClient


class CyclonesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCyclonesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._forecasts: typing.Optional[ForecastsClient] = None

    @property
    def with_raw_response(self) -> RawCyclonesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCyclonesClient
        """
        return self._raw_client

    @property
    def forecasts(self):
        if self._forecasts is None:
            from .forecasts.client import ForecastsClient  # noqa: E402

            self._forecasts = ForecastsClient(client_wrapper=self._client_wrapper)
        return self._forecasts


class AsyncCyclonesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCyclonesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._forecasts: typing.Optional[AsyncForecastsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawCyclonesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCyclonesClient
        """
        return self._raw_client

    @property
    def forecasts(self):
        if self._forecasts is None:
            from .forecasts.client import AsyncForecastsClient  # noqa: E402

            self._forecasts = AsyncForecastsClient(client_wrapper=self._client_wrapper)
        return self._forecasts



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.datetime_utils import serialize_datetime
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.feature_collection import FeatureCollection
from ..types.http_validation_error import HttpValidationError
from ..types.time_series_feature_collection_model import TimeSeriesFeatureCollectionModel


class RawPortfoliosClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def features(
        self,
        portfolio_id: str,
        *,
        x: typing.Optional[int] = None,
        y: typing.Optional[int] = None,
        z: typing.Optional[int] = None,
        country: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[FeatureCollection]:
        """
        Return a GeoJSON FeatureCollection of features for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        x : typing.Optional[int]

        y : typing.Optional[int]

        z : typing.Optional[int]

        country : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FeatureCollection]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"portfolios/{jsonable_encoder(portfolio_id)}/features",
            method="GET",
            params={
                "x": x,
                "y": y,
                "z": z,
                "country": country,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FeatureCollection,
                    parse_obj_as(
                        type_=FeatureCollection,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def forecasts(
        self,
        portfolio_id: str,
        *,
        init_time: dt.datetime,
        id: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        variables: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TimeSeriesFeatureCollectionModel]:
        """
        Return a GeoJSON FeatureCollection of forecasts for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        init_time : dt.datetime

        id : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        variables : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TimeSeriesFeatureCollectionModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"portfolios/{jsonable_encoder(portfolio_id)}/forecasts",
            method="GET",
            params={
                "id": id,
                "variables": variables,
                "init_time": serialize_datetime(init_time),
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TimeSeriesFeatureCollectionModel,
                    parse_obj_as(
                        type_=TimeSeriesFeatureCollectionModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def observations(
        self,
        portfolio_id: str,
        *,
        valid_time_start: dt.datetime,
        valid_time_end: dt.datetime,
        id: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        variables: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TimeSeriesFeatureCollectionModel]:
        """
        Return a GeoJSON FeatureCollection of observations for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        valid_time_start : dt.datetime

        valid_time_end : dt.datetime

        id : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        variables : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TimeSeriesFeatureCollectionModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"portfolios/{jsonable_encoder(portfolio_id)}/observations",
            method="GET",
            params={
                "id": id,
                "variables": variables,
                "valid_time_start": serialize_datetime(valid_time_start),
                "valid_time_end": serialize_datetime(valid_time_end),
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TimeSeriesFeatureCollectionModel,
                    parse_obj_as(
                        type_=TimeSeriesFeatureCollectionModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def init_time(
        self, portfolio_id: str, *, time: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[dt.datetime]:
        """
        Return the latest initialization time for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        time : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[dt.datetime]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"portfolios/{jsonable_encoder(portfolio_id)}/init_time",
            method="GET",
            params={
                "time": serialize_datetime(time),
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    dt.datetime,
                    parse_obj_as(
                        type_=dt.datetime,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPortfoliosClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def features(
        self,
        portfolio_id: str,
        *,
        x: typing.Optional[int] = None,
        y: typing.Optional[int] = None,
        z: typing.Optional[int] = None,
        country: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[FeatureCollection]:
        """
        Return a GeoJSON FeatureCollection of features for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        x : typing.Optional[int]

        y : typing.Optional[int]

        z : typing.Optional[int]

        country : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FeatureCollection]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"portfolios/{jsonable_encoder(portfolio_id)}/features",
            method="GET",
            params={
                "x": x,
                "y": y,
                "z": z,
                "country": country,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FeatureCollection,
                    parse_obj_as(
                        type_=FeatureCollection,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def forecasts(
        self,
        portfolio_id: str,
        *,
        init_time: dt.datetime,
        id: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        variables: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TimeSeriesFeatureCollectionModel]:
        """
        Return a GeoJSON FeatureCollection of forecasts for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        init_time : dt.datetime

        id : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        variables : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TimeSeriesFeatureCollectionModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"portfolios/{jsonable_encoder(portfolio_id)}/forecasts",
            method="GET",
            params={
                "id": id,
                "variables": variables,
                "init_time": serialize_datetime(init_time),
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TimeSeriesFeatureCollectionModel,
                    parse_obj_as(
                        type_=TimeSeriesFeatureCollectionModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def observations(
        self,
        portfolio_id: str,
        *,
        valid_time_start: dt.datetime,
        valid_time_end: dt.datetime,
        id: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        variables: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TimeSeriesFeatureCollectionModel]:
        """
        Return a GeoJSON FeatureCollection of observations for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        valid_time_start : dt.datetime

        valid_time_end : dt.datetime

        id : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        variables : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TimeSeriesFeatureCollectionModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"portfolios/{jsonable_encoder(portfolio_id)}/observations",
            method="GET",
            params={
                "id": id,
                "variables": variables,
                "valid_time_start": serialize_datetime(valid_time_start),
                "valid_time_end": serialize_datetime(valid_time_end),
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TimeSeriesFeatureCollectionModel,
                    parse_obj_as(
                        type_=TimeSeriesFeatureCollectionModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def init_time(
        self, portfolio_id: str, *, time: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[dt.datetime]:
        """
        Return the latest initialization time for a portfolio.

        Parameters
        ----------
        portfolio_id : str

        time : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[dt.datetime]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"portfolios/{jsonable_encoder(portfolio_id)}/init_time",
            method="GET",
            params={
                "time": serialize_datetime(time),
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    dt.datetime,
                    parse_obj_as(
                        type_=dt.datetime,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.hourly_weather_response import HourlyWeatherResponse
from ...types.timezone import Timezone
from ...types.units import Units
from .raw_client import AsyncRawExperimentalClient, RawExperimentalClient

if typing.TYPE_CHECKING:
    from .past.client import AsyncPastClient, PastClient
    from .personalized.client import AsyncPersonalizedClient, PersonalizedClient
    from .regional.client import AsyncRegionalClient, RegionalClient


class ExperimentalClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawExperimentalClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._regional: typing.Optional[RegionalClient] = None
        self._personalized: typing.Optional[PersonalizedClient] = None
        self._past: typing.Optional[PastClient] = None

    @property
    def with_raw_response(self) -> RawExperimentalClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawExperimentalClient
        """
        return self._raw_client

    def extended(
        self,
        *,
        latitude: float,
        longitude: float,
        timezone: typing.Optional[Timezone] = None,
        units: typing.Optional[Units] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HourlyWeatherResponse:
        """
        Get hourly weather forecast for a specific location and time

        Parameters
        ----------
        latitude : float

        longitude : float

        timezone : typing.Optional[Timezone]

        units : typing.Optional[Units]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HourlyWeatherResponse
            Successful Response

        Examples
        --------
        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.weather.experimental.extended(
            latitude=47.6061,
            longitude=-122.3328,
            timezone="local",
            units="metric",
        )
        """
        _response = self._raw_client.extended(
            latitude=latitude, longitude=longitude, timezone=timezone, units=units, request_options=request_options
        )
        return _response.data

    @property
    def regional(self):
        if self._regional is None:
            from .regional.client import RegionalClient  # noqa: E402

            self._regional = RegionalClient(client_wrapper=self._client_wrapper)
        return self._regional

    @property
    def personalized(self):
        if self._personalized is None:
            from .personalized.client import PersonalizedClient  # noqa: E402

            self._personalized = PersonalizedClient(client_wrapper=self._client_wrapper)
        return self._personalized

    @property
    def past(self):
        if self._past is None:
            from .past.client import PastClient  # noqa: E402

            self._past = PastClient(client_wrapper=self._client_wrapper)
        return self._past


class AsyncExperimentalClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawExperimentalClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._regional: typing.Optional[AsyncRegionalClient] = None
        self._personalized: typing.Optional[AsyncPersonalizedClient] = None
        self._past: typing.Optional[AsyncPastClient] = None

    @property
    def with_raw_response(self) -> AsyncRawExperimentalClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawExperimentalClient
        """
        return self._raw_client

    async def extended(
        self,
        *,
        latitude: float,
        longitude: float,
        timezone: typing.Optional[Timezone] = None,
        units: typing.Optional[Units] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HourlyWeatherResponse:
        """
        Get hourly weather forecast for a specific location and time

        Parameters
        ----------
        latitude : float

        longitude : float

        timezone : typing.Optional[Timezone]

        units : typing.Optional[Units]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HourlyWeatherResponse
            Successful Response

        Examples
        --------
        import asyncio

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.weather.experimental.extended(
                latitude=47.6061,
                longitude=-122.3328,
                timezone="local",
                units="metric",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.extended(
            latitude=latitude, longitude=longitude, timezone=timezone, units=units, request_options=request_options
        )
        return _response.data

    @property
    def regional(self):
        if self._regional is None:
            from .regional.client import AsyncRegionalClient  # noqa: E402

            self._regional = AsyncRegionalClient(client_wrapper=self._client_wrapper)
        return self._regional

    @property
    def personalized(self):
        if self._personalized is None:
            from .personalized.client import AsyncPersonalizedClient  # noqa: E402

            self._personalized = AsyncPersonalizedClient(client_wrapper=self._client_wrapper)
        return self._personalized

    @property
    def past(self):
        if self._past is None:
            from .past.client import AsyncPastClient  # noqa: E402

            self._past = AsyncPastClient(client_wrapper=self._client_wrapper)
        return self._past



# This file was auto-generated by Fern from our API Definition.

import typing

from .imperial_units import ImperialUnits
from .metric_units import MetricUnits

HourlyWeatherResponseUnits = typing.Union[MetricUnits, ImperialUnits]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .base_model import BaseModel
    from .cyclone_forecast_response import CycloneForecastResponse
    from .daily_conditions import DailyConditions
    from .daily_weather_response import DailyWeatherResponse
    from .daily_weather_response_units import DailyWeatherResponseUnits
    from .feature import Feature
    from .feature_collection import FeatureCollection
    from .feature_geometry import (
        FeatureGeometry,
        FeatureGeometry_GeometryCollection,
        FeatureGeometry_LineString,
        FeatureGeometry_MultiLineString,
        FeatureGeometry_MultiPoint,
        FeatureGeometry_MultiPolygon,
        FeatureGeometry_Point,
        FeatureGeometry_Polygon,
    )
    from .feature_id import FeatureId
    from .feature_properties import FeatureProperties
    from .geo_location import GeoLocation
    from .geometry_collection import GeometryCollection
    from .geometry_collection_geometries_item import (
        GeometryCollectionGeometriesItem,
        GeometryCollectionGeometriesItem_GeometryCollection,
        GeometryCollectionGeometriesItem_LineString,
        GeometryCollectionGeometriesItem_MultiLineString,
        GeometryCollectionGeometriesItem_MultiPoint,
        GeometryCollectionGeometriesItem_MultiPolygon,
        GeometryCollectionGeometriesItem_Point,
        GeometryCollectionGeometriesItem_Polygon,
    )
    from .gftus_base_units import GftusBaseUnits
    from .gftus_hourly_conditions import GftusHourlyConditions
    from .gftus_hourly_weather_response import GftusHourlyWeatherResponse
    from .hourly_conditions import HourlyConditions
    from .hourly_weather_response import HourlyWeatherResponse
    from .hourly_weather_response_units import HourlyWeatherResponseUnits
    from .http_validation_error import HttpValidationError
    from .imperial_units import ImperialUnits
    from .line_string import LineString
    from .line_string_coordinates_item import LineStringCoordinatesItem
    from .metric_units import MetricUnits
    from .model_name import ModelName
    from .multi_line_string import MultiLineString
    from .multi_line_string_coordinates_item_item import MultiLineStringCoordinatesItemItem
    from .multi_point import MultiPoint
    from .multi_point_coordinates_item import MultiPointCoordinatesItem
    from .multi_polygon import MultiPolygon
    from .multi_polygon_coordinates_item_item_item import MultiPolygonCoordinatesItemItemItem
    from .point import Point
    from .point_coordinates import PointCoordinates
    from .polygon import Polygon
    from .polygon_coordinates_item_item import PolygonCoordinatesItemItem
    from .position_2_d import Position2D
    from .position_3_d import Position3D
    from .precipitation_type import PrecipitationType
    from .time_series_feature_collection_model import TimeSeriesFeatureCollectionModel
    from .time_series_feature_model import TimeSeriesFeatureModel
    from .time_series_feature_model_geometry import (
        TimeSeriesFeatureModelGeometry,
        TimeSeriesFeatureModelGeometry_GeometryCollection,
        TimeSeriesFeatureModelGeometry_LineString,
        TimeSeriesFeatureModelGeometry_MultiLineString,
        TimeSeriesFeatureModelGeometry_MultiPoint,
        TimeSeriesFeatureModelGeometry_MultiPolygon,
        TimeSeriesFeatureModelGeometry_Point,
        TimeSeriesFeatureModelGeometry_Polygon,
    )
    from .time_series_feature_model_id import TimeSeriesFeatureModelId
    from .time_series_properties_model import TimeSeriesPropertiesModel
    from .time_series_properties_model_static import TimeSeriesPropertiesModelStatic
    from .timezone import Timezone
    from .units import Units
    from .validation_error import ValidationError
    from .validation_error_loc_item import ValidationErrorLocItem
    from .weather_code import WeatherCode
_dynamic_imports: typing.Dict[str, str] = {
    "BaseModel": ".base_model",
    "CycloneForecastResponse": ".cyclone_forecast_response",
    "DailyConditions": ".daily_conditions",
    "DailyWeatherResponse": ".daily_weather_response",
    "DailyWeatherResponseUnits": ".daily_weather_response_units",
    "Feature": ".feature",
    "FeatureCollection": ".feature_collection",
    "FeatureGeometry": ".feature_geometry",
    "FeatureGeometry_GeometryCollection": ".feature_geometry",
    "FeatureGeometry_LineString": ".feature_geometry",
    "FeatureGeometry_MultiLineString": ".feature_geometry",
    "FeatureGeometry_MultiPoint": ".feature_geometry",
    "FeatureGeometry_MultiPolygon": ".feature_geometry",
    "FeatureGeometry_Point": ".feature_geometry",
    "FeatureGeometry_Polygon": ".feature_geometry",
    "FeatureId": ".feature_id",
    "FeatureProperties": ".feature_properties",
    "GeoLocation": ".geo_location",
    "GeometryCollection": ".geometry_collection",
    "GeometryCollectionGeometriesItem": ".geometry_collection_geometries_item",
    "GeometryCollectionGeometriesItem_GeometryCollection": ".geometry_collection_geometries_item",
    "GeometryCollectionGeometriesItem_LineString": ".geometry_collection_geometries_item",
    "GeometryCollectionGeometriesItem_MultiLineString": ".geometry_collection_geometries_item",
    "GeometryCollectionGeometriesItem_MultiPoint": ".geometry_collection_geometries_item",
    "GeometryCollectionGeometriesItem_MultiPolygon": ".geometry_collection_geometries_item",
    "GeometryCollectionGeometriesItem_Point": ".geometry_collection_geometries_item",
    "GeometryCollectionGeometriesItem_Polygon": ".geometry_collection_geometries_item",
    "GftusBaseUnits": ".gftus_base_units",
    "GftusHourlyConditions": ".gftus_hourly_conditions",
    "GftusHourlyWeatherResponse": ".gftus_hourly_weather_response",
    "HourlyConditions": ".hourly_conditions",
    "HourlyWeatherResponse": ".hourly_weather_response",
    "HourlyWeatherResponseUnits": ".hourly_weather_response_units",
    "HttpValidationError": ".http_validation_error",
    "ImperialUnits": ".imperial_units",
    "LineString": ".line_string",
    "LineStringCoordinatesItem": ".line_string_coordinates_item",
    "MetricUnits": ".metric_units",
    "ModelName": ".model_name",
    "MultiLineString": ".multi_line_string",
    "MultiLineStringCoordinatesItemItem": ".multi_line_string_coordinates_item_item",
    "MultiPoint": ".multi_point",
    "MultiPointCoordinatesItem": ".multi_point_coordinates_item",
    "MultiPolygon": ".multi_polygon",
    "MultiPolygonCoordinatesItemItemItem": ".multi_polygon_coordinates_item_item_item",
    "Point": ".point",
    "PointCoordinates": ".point_coordinates",
    "Polygon": ".polygon",
    "PolygonCoordinatesItemItem": ".polygon_coordinates_item_item",
    "Position2D": ".position_2_d",
    "Position3D": ".position_3_d",
    "PrecipitationType": ".precipitation_type",
    "TimeSeriesFeatureCollectionModel": ".time_series_feature_collection_model",
    "TimeSeriesFeatureModel": ".time_series_feature_model",
    "TimeSeriesFeatureModelGeometry": ".time_series_feature_model_geometry",
    "TimeSeriesFeatureModelGeometry_GeometryCollection": ".time_series_feature_model_geometry",
    "TimeSeriesFeatureModelGeometry_LineString": ".time_series_feature_model_geometry",
    "TimeSeriesFeatureModelGeometry_MultiLineString": ".time_series_feature_model_geometry",
    "TimeSeriesFeatureModelGeometry_MultiPoint": ".time_series_feature_model_geometry",
    "TimeSeriesFeatureModelGeometry_MultiPolygon": ".time_series_feature_model_geometry",
    "TimeSeriesFeatureModelGeometry_Point": ".time_series_feature_model_geometry",
    "TimeSeriesFeatureModelGeometry_Polygon": ".time_series_feature_model_geometry",
    "TimeSeriesFeatureModelId": ".time_series_feature_model_id",
    "TimeSeriesPropertiesModel": ".time_series_properties_model",
    "TimeSeriesPropertiesModelStatic": ".time_series_properties_model_static",
    "Timezone": ".timezone",
    "Units": ".units",
    "ValidationError": ".validation_error",
    "ValidationErrorLocItem": ".validation_error_loc_item",
    "WeatherCode": ".weather_code",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BaseModel",
    "CycloneForecastResponse",
    "DailyConditions",
    "DailyWeatherResponse",
    "DailyWeatherResponseUnits",
    "Feature",
    "FeatureCollection",
    "FeatureGeometry",
    "FeatureGeometry_GeometryCollection",
    "FeatureGeometry_LineString",
    "FeatureGeometry_MultiLineString",
    "FeatureGeometry_MultiPoint",
    "FeatureGeometry_MultiPolygon",
    "FeatureGeometry_Point",
    "FeatureGeometry_Polygon",
    "FeatureId",
    "FeatureProperties",
    "GeoLocation",
    "GeometryCollection",
    "GeometryCollectionGeometriesItem",
    "GeometryCollectionGeometriesItem_GeometryCollection",
    "GeometryCollectionGeometriesItem_LineString",
    "GeometryCollectionGeometriesItem_MultiLineString",
    "GeometryCollectionGeometriesItem_MultiPoint",
    "GeometryCollectionGeometriesItem_MultiPolygon",
    "GeometryCollectionGeometriesItem_Point",
    "GeometryCollectionGeometriesItem_Polygon",
    "GftusBaseUnits",
    "GftusHourlyConditions",
    "GftusHourlyWeatherResponse",
    "HourlyConditions",
    "HourlyWeatherResponse",
    "HourlyWeatherResponseUnits",
    "HttpValidationError",
    "ImperialUnits",
    "LineString",
    "LineStringCoordinatesItem",
    "MetricUnits",
    "ModelName",
    "MultiLineString",
    "MultiLineStringCoordinatesItemItem",
    "MultiPoint",
    "MultiPointCoordinatesItem",
    "MultiPolygon",
    "MultiPolygonCoordinatesItemItemItem",
    "Point",
    "PointCoordinates",
    "Polygon",
    "PolygonCoordinatesItemItem",
    "Position2D",
    "Position3D",
    "PrecipitationType",
    "TimeSeriesFeatureCollectionModel",
    "TimeSeriesFeatureModel",
    "TimeSeriesFeatureModelGeometry",
    "TimeSeriesFeatureModelGeometry_GeometryCollection",
    "TimeSeriesFeatureModelGeometry_LineString",
    "TimeSeriesFeatureModelGeometry_MultiLineString",
    "TimeSeriesFeatureModelGeometry_MultiPoint",
    "TimeSeriesFeatureModelGeometry_MultiPolygon",
    "TimeSeriesFeatureModelGeometry_Point",
    "TimeSeriesFeatureModelGeometry_Polygon",
    "TimeSeriesFeatureModelId",
    "TimeSeriesPropertiesModel",
    "TimeSeriesPropertiesModelStatic",
    "Timezone",
    "Units",
    "ValidationError",
    "ValidationErrorLocItem",
    "WeatherCode",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from .raw_client import AsyncRawPersonalizedClient, RawPersonalizedClient


class PersonalizedClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPersonalizedClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPersonalizedClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPersonalizedClient
        """
        return self._raw_client

    def total_energies(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from silurian import Earth

        client = Earth(
            api_key="YOUR_API_KEY",
        )
        client.weather.experimental.personalized.total_energies()
        """
        _response = self._raw_client.total_energies(request_options=request_options)
        return _response.data


class AsyncPersonalizedClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPersonalizedClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPersonalizedClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPersonalizedClient
        """
        return self._raw_client

    async def total_energies(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from silurian import AsyncEarth

        client = AsyncEarth(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.weather.experimental.personalized.total_energies()


        asyncio.run(main())
        """
        _response = await self._raw_client.total_energies(request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawWeatherClient, RawWeatherClient

if typing.TYPE_CHECKING:
    from .experimental.client import AsyncExperimentalClient, ExperimentalClient
    from .forecast.client import AsyncForecastClient, ForecastClient
    from .past.client import AsyncPastClient, PastClient


class WeatherClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWeatherClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._forecast: typing.Optional[ForecastClient] = None
        self._experimental: typing.Optional[ExperimentalClient] = None
        self._past: typing.Optional[PastClient] = None

    @property
    def with_raw_response(self) -> RawWeatherClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWeatherClient
        """
        return self._raw_client

    @property
    def forecast(self):
        if self._forecast is None:
            from .forecast.client import ForecastClient  # noqa: E402

            self._forecast = ForecastClient(client_wrapper=self._client_wrapper)
        return self._forecast

    @property
    def experimental(self):
        if self._experimental is None:
            from .experimental.client import ExperimentalClient  # noqa: E402

            self._experimental = ExperimentalClient(client_wrapper=self._client_wrapper)
        return self._experimental

    @property
    def past(self):
        if self._past is None:
            from .past.client import PastClient  # noqa: E402

            self._past = PastClient(client_wrapper=self._client_wrapper)
        return self._past


class AsyncWeatherClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWeatherClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._forecast: typing.Optional[AsyncForecastClient] = None
        self._experimental: typing.Optional[AsyncExperimentalClient] = None
        self._past: typing.Optional[AsyncPastClient] = None

    @property
    def with_raw_response(self) -> AsyncRawWeatherClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWeatherClient
        """
        return self._raw_client

    @property
    def forecast(self):
        if self._forecast is None:
            from .forecast.client import AsyncForecastClient  # noqa: E402

            self._forecast = AsyncForecastClient(client_wrapper=self._client_wrapper)
        return self._forecast

    @property
    def experimental(self):
        if self._experimental is None:
            from .experimental.client import AsyncExperimentalClient  # noqa: E402

            self._experimental = AsyncExperimentalClient(client_wrapper=self._client_wrapper)
        return self._experimental

    @property
    def past(self):
        if self._past is None:
            from .past.client import AsyncPastClient  # noqa: E402

            self._past = AsyncPastClient(client_wrapper=self._client_wrapper)
        return self._past
